@@ -30,15 +30,20 @@
 #include <fcntl.h>
 #include <signal.h>
 #include <string.h>
+#include <stdlib.h>   /* for fdwalk */
+#include <dirent.h>
 
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif /* HAVE_SYS_SELECT_H */
 
-#include "glib.h"
-#include "galias.h"
+#ifdef HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif /* HAVE_SYS_RESOURCE_H */
 
+#include "glib.h"
 #include "glibintl.h"
+#include "galias.h"
 
 static gint g_execute (const gchar  *file,
                        gchar **argv,
@@ -85,6 +90,13 @@ g_spawn_error_quark (void)
  * See g_spawn_async_with_pipes() for a full description; this function
  * simply calls the g_spawn_async_with_pipes() without any pipes.
  * 
+ * <note><para>
+ * If you are writing a GTK+ application, and the program you 
+ * are spawning is a graphical application, too, then you may
+ * want to use gdk_spawn_on_screen() instead to ensure that 
+ * the spawned program opens its windows on the right screen.
+ * </para></note>
+ *
  * Return value: %TRUE on success, %FALSE if error is set
  **/
 gboolean
@@ -129,6 +141,9 @@ close_and_invalidate (gint *fd)
   return ret;
 }
 
+/* Some versions of OS X define READ_OK in public headers */
+#undef READ_OK
+
 typedef enum
 {
   READ_FAILED = 0, /* FALSE */
@@ -519,7 +534,7 @@ g_spawn_sync (const gchar          *working_directory,
  *
  * If non-%NULL, @child_pid will on Unix be filled with the child's
  * process ID. You can use the process ID to send signals to the
- * child, or to waitpid() if you specified the
+ * child, or to use g_child_watch_add() (or waitpid()) if you specified the
  * %G_SPAWN_DO_NOT_REAP_CHILD flag. On Windows, @child_pid will be
  * filled with a handle to the child process only if you specified the
  * %G_SPAWN_DO_NOT_REAP_CHILD flag. You can then access the child
@@ -535,14 +550,16 @@ g_spawn_sync (const gchar          *working_directory,
  * when they are no longer in use. If these parameters are %NULL, the corresponding
  * pipe won't be created.
  *
- * If @standard_input is NULL, the child's standard input is attached to /dev/null
- * unless %G_SPAWN_CHILD_INHERITS_STDIN is set.
+ * If @standard_input is NULL, the child's standard input is attached to 
+ * /dev/null unless %G_SPAWN_CHILD_INHERITS_STDIN is set.
  *
- * If @standard_error is NULL, the child's standard error goes to the same location
- * as the parent's standard error unless %G_SPAWN_STDERR_TO_DEV_NULL is set.
+ * If @standard_error is NULL, the child's standard error goes to the same 
+ * location as the parent's standard error unless %G_SPAWN_STDERR_TO_DEV_NULL 
+ * is set.
  *
- * If @standard_output is NULL, the child's standard output goes to the same location
- * as the parent's standard output unless %G_SPAWN_STDOUT_TO_DEV_NULL is set.
+ * If @standard_output is NULL, the child's standard output goes to the same 
+ * location as the parent's standard output unless %G_SPAWN_STDOUT_TO_DEV_NULL 
+ * is set.
  *
  * @error can be %NULL to ignore errors, or non-%NULL to report errors.
  * If an error is set, the function returns %FALSE. Errors
@@ -556,6 +573,13 @@ g_spawn_sync (const gchar          *working_directory,
  *
  * If @child_pid is not %NULL and an error does not occur then the returned
  * pid must be closed using g_spawn_close_pid().
+ *
+ * <note><para>
+ * If you are writing a GTK+ application, and the program you 
+ * are spawning is a graphical application, too, then you may
+ * want to use gdk_spawn_on_screen_with_pipes() instead to ensure that 
+ * the spawned program opens its windows no the right screen.
+ * </para></note>
  * 
  * Return value: %TRUE on success, %FALSE if an error was set
  **/
@@ -850,12 +874,82 @@ write_err_and_exit (gint fd, gint msg)
   _exit (1);
 }
 
-static void
-set_cloexec (gint fd)
+static int
+set_cloexec (void *data, gint fd)
 {
-  fcntl (fd, F_SETFD, FD_CLOEXEC);
+  if (fd >= GPOINTER_TO_INT (data))
+    fcntl (fd, F_SETFD, FD_CLOEXEC);
+
+  return 0;
 }
 
+#ifndef HAVE_FDWALK
+static int
+fdwalk (int (*cb)(void *data, int fd), void *data)
+{
+  gint open_max;
+  gint fd;
+  gint res = 0;
+  
+#ifdef HAVE_SYS_RESOURCE_H
+  struct rlimit rl;
+#endif
+
+#ifdef __linux__  
+  DIR *d;
+
+  if ((d = opendir("/proc/self/fd"))) {
+      struct dirent *de;
+
+      while ((de = readdir(d))) {
+          glong l;
+          gchar *e = NULL;
+
+          if (de->d_name[0] == '.')
+              continue;
+            
+          errno = 0;
+          l = strtol(de->d_name, &e, 10);
+          if (errno != 0 || !e || *e)
+              continue;
+
+          fd = (gint) l;
+
+          if ((glong) fd != l)
+              continue;
+
+          if (fd == dirfd(d))
+              continue;
+
+          if ((res = cb (data, fd)) != 0)
+              break;
+        }
+      
+      closedir(d);
+      return res;
+  }
+
+  /* If /proc is not mounted or not accessible we fall back to the old
+   * rlimit trick */
+
+#endif
+  
+#ifdef HAVE_SYS_RESOURCE_H
+      
+  if (getrlimit(RLIMIT_NOFILE, &rl) == 0)
+      open_max = rl.rlim_max;
+  else
+#endif
+      open_max = sysconf (_SC_OPEN_MAX);
+
+  for (fd = 0; fd < open_max; fd++)
+      if ((res = cb (data, fd)) != 0)
+          break;
+
+  return res;
+}
+#endif
+
 static gint
 sane_dup2 (gint fd1, gint fd2)
 {
@@ -905,17 +999,12 @@ do_exec (gint                  child_err_report_fd,
    */
   if (close_descriptors)
     {
-      gint open_max;
-      gint i;
-      
-      open_max = sysconf (_SC_OPEN_MAX);
-      for (i = 3; i < open_max; i++)
-        set_cloexec (i);
+      fdwalk (set_cloexec, GINT_TO_POINTER(3));
     }
   else
     {
       /* We need to do child_err_report_fd anyway */
-      set_cloexec (child_err_report_fd);
+      set_cloexec (GINT_TO_POINTER(0), child_err_report_fd);
     }
   
   /* Redirect pipes as required */
@@ -1442,8 +1531,8 @@ g_execute (const gchar *file,
       gboolean got_eacces = 0;
       const gchar *path, *p;
       gchar *name, *freeme;
-      size_t len;
-      size_t pathlen;
+      gsize len;
+      gsize pathlen;
 
       path = g_getenv ("PATH");
       if (path == NULL)
