@@ -16,8 +16,8 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#ifndef __G_NOTIFY_H__
-#define __G_NOTIFY_H__
+#ifndef __G_OBJECT_NOTIFY_QUEUE_H__
+#define __G_OBJECT_NOTIFY_QUEUE_H__
 
 #include        <string.h> /* memset */
 #include        <glib-object.h>
@@ -38,14 +38,17 @@ struct _GObjectNotifyContext
 {
   GQuark                       quark_notify_queue;
   GObjectNotifyQueueDispatcher dispatcher;
-  GTrashStack                 *nqueue_trash;
+  GTrashStack                 *_nqueue_trash; /* unused */
 };
 struct _GObjectNotifyQueue
 {
   GObjectNotifyContext *context;
   GSList               *pspecs;
-  guint                 n_pspecs;
-  guint                 freeze_count;
+  guint16               n_pspecs;
+  guint16               freeze_count;
+  /* currently, this structure abuses the GList allocation chain and thus
+   * must be <= sizeof (GList)
+   */
 };
 
 
@@ -56,7 +59,7 @@ g_object_notify_queue_free (gpointer data)
   GObjectNotifyQueue *nqueue = data;
 
   g_slist_free (nqueue->pspecs);
-  g_trash_stack_push (&nqueue->context->nqueue_trash, nqueue);
+  g_list_free_1 ((void*) nqueue);
 }
 
 static inline GObjectNotifyQueue*
@@ -68,20 +71,14 @@ g_object_notify_queue_freeze (GObject		   *object,
   nqueue = g_datalist_id_get_data (&object->qdata, context->quark_notify_queue);
   if (!nqueue)
     {
-      nqueue = g_trash_stack_pop (&context->nqueue_trash);
-      if (!nqueue)
-	{
-	  guint i;
-
-	  nqueue = g_new (GObjectNotifyQueue, 16);
-	  for (i = 0; i < 15; i++)
-	    g_trash_stack_push (&context->nqueue_trash, nqueue++);
-	}
+      nqueue = (void*) g_list_alloc ();
       memset (nqueue, 0, sizeof (*nqueue));
       nqueue->context = context;
       g_datalist_id_set_data_full (&object->qdata, context->quark_notify_queue,
 				   nqueue, g_object_notify_queue_free);
     }
+
+  g_return_val_if_fail (nqueue->freeze_count < 65535, nqueue);
   nqueue->freeze_count++;
 
   return nqueue;
@@ -109,7 +106,7 @@ g_object_notify_queue_thaw (GObject            *object,
   for (slist = nqueue->pspecs; slist; slist = slist->next)
     {
       GParamSpec *pspec = slist->data;
-      gint i = 0;
+      guint i = 0;
 
       /* dedup, make pspecs in the list unique */
     redo_dedup_check:
@@ -145,6 +142,14 @@ g_object_notify_queue_add (GObject            *object,
 {
   if (pspec->flags & G_PARAM_READABLE)
     {
+      GParamSpec *redirect;
+
+      g_return_if_fail (nqueue->n_pspecs < 65535);
+
+      redirect = g_param_spec_get_redirect_target (pspec);
+      if (redirect)
+	pspec = redirect;
+	    
       /* we do the deduping in _thaw */
       nqueue->pspecs = g_slist_prepend (nqueue->pspecs, pspec);
       nqueue->n_pspecs++;
@@ -161,4 +166,4 @@ g_object_notify_queue_from_object (GObject              *object,
 
 G_END_DECLS
 
-#endif /* __G_OBJECT_H__ */
+#endif /* __G_OBJECT_NOTIFY_QUEUE_H__ */
